#define LGFX_USE_V1

#include <SPI.h>
#include <SD.h>
#include <FS.h>
#include <LovyanGFX.hpp>
#include <Ticker.h>          //Call the ticker. H Library
Ticker ticker1;
#include <U8g2lib.h>
#ifdef U8X8_HAVE_HW_I2C
#include <Wire.h>
#include <FT6236.h>


#endif
U8G2_SSD1306_128X32_UNIVISION_F_SW_I2C  u8g2(U8G2_R0, /* clock=*/ 39, /* data=*/ 38, /* reset=*/ U8X8_PIN_NONE);   // Adafruit Feather ESP8266/32u4 Boards + FeatherWing OLED

#define I2C_SCL 39
#define I2C_SDA 38

const int i2c_touch_addr = TOUCH_I2C_ADD;
#define get_pos ft6236_pos

#define SPI_MOSI 2 //1
#define SPI_MISO 41
#define SPI_SCK 42
#define SD_CS 1 //2

#define LCD_CS -1
#define LCD_BLK 46


#define G4 392
#define A4 440
#define B4 494
#define C5 523
#define D5 587
#define E5 659
#define F5 698
#define G5 784
int Tone[] = { B4, B4, B4, B4, B4, B4,
               B4, D5, G4, A4, B4,
               C5, C5, C5, C5, C5, B4, B4, B4,
               D5, D5, C5, A4, G4, G5
             };
class LGFX : public lgfx::LGFX_Device
{
    //lgfx::Panel_ILI9341 _panel_instance;
    lgfx::Panel_ILI9488 _panel_instance;
    lgfx::Bus_Parallel16 _bus_instance; 

  public:
    
    LGFX(void)
    {
      {
        auto cfg = _bus_instance.config(); 

        
        cfg.port = 0; 
        cfg.freq_write = 20000000; 
        cfg.pin_wr = 18; 
        cfg.pin_rd = 48; 
        cfg.pin_rs = 45; 

        cfg.pin_d0 = 47;
        cfg.pin_d1 = 21;
        cfg.pin_d2 = 14;
        cfg.pin_d3 = 13;
        cfg.pin_d4 = 12;
        cfg.pin_d5 = 11;
        cfg.pin_d6 = 10;
        cfg.pin_d7 = 9;
        cfg.pin_d8 = 3;
        cfg.pin_d9 = 8;
        cfg.pin_d10 = 16;
        cfg.pin_d11 = 15;
        cfg.pin_d12 = 7;
        cfg.pin_d13 = 6;
        cfg.pin_d14 = 5;
        cfg.pin_d15 = 4;

        _bus_instance.config(cfg);              
        _panel_instance.setBus(&_bus_instance);
      }

      { 
        auto cfg = _panel_instance.config(); 

        cfg.pin_cs = -1; 
        cfg.pin_rst = -1; 
        cfg.pin_busy = -1; 
        

        cfg.memory_width = 320; 
        cfg.memory_height = 480; 
        cfg.panel_width = 320; 
        cfg.panel_height = 480;
        cfg.offset_x = 0; 
        cfg.offset_y = 0; 
        cfg.offset_rotation = 0; 
        cfg.dummy_read_pixel = 8; 
        cfg.dummy_read_bits = 1; 
        cfg.readable = true; 
        cfg.invert = false;
        cfg.rgb_order = false; 
        cfg.dlen_16bit = true; 
        cfg.bus_shared = true; // 与SD卡共享总线时设置为真（通过drawJpgFile等进行总线控制）
        _panel_instance.config(cfg);
      }

      setPanel(&_panel_instance); // 设置要使用的面板。
    }
};

void callback1()  //Callback function
{
  //IO40
  digitalWrite(40, HIGH);
  delay(500);
  digitalWrite(40, LOW);
  delay(500);
}
bool connected_state = false;   

LGFX lcd;
int pos[2] = {0, 0};
int err_code = 0;
char LU[10];
char B[10];
int i = 0;
int rect_length = 40;
char CloseData;
int NO_Test_Flag = 0;
int Test_Flag = 0;
int Close_Flag = 0;
int wifi_close_flag = 0;

char UWB;
char *uwb_info[128] = {};
char uwb_buf[128] = {};
int uwb_bufindex = 0;
int uwb_flag = 0;
char buf[128] = {};
int bufindex = 0;
int first_Flag = 0;
char wifi_account[20] = {}, wifi_password[20] = {} , dtm[100] = {};
char *info[128] = {};
int wifi_flag = 0;


int print_img(fs::FS & fs, String filename, int x, int y)
{
  File f = fs.open(filename, "r");
  if (!f)
  {
    Serial.println("Failed to open file for reading");
    f.close();
    return 0;
  }

  f.seek(54);
  int X = x;
  int Y = y;
  uint8_t RGB[3 * X];
  for (int row = 0; row < Y; row++)
  {
    f.seek(54 + 3 * X * row);
    f.read(RGB, 3 * X);
    lcd.pushImage(0, row, X, 1, (lgfx::rgb888_t *)RGB);
  }

  f.close();
  return 0;
}


//BMP is 32 bit...
int print_img_small(fs::FS & fs, String filename, int x, int y, int w, int h)
{
  File f = fs.open(filename, "r");
  if (!f)
  {
    Serial.println("Failed to open file for reading");
    f.close();
    return 0;
  }

  f.seek(54);
  int X = w;
  int Y = h;
  uint8_t RGB[3 * X];
  uint8_t RGBA[4 * X];
  for (int row = 0; row < Y; row++)
  {
    f.seek(54 + 4 * X * row);
    f.read(RGBA, 4 * X);

    for (int i = 0; i < X; i++)
    {
      RGB[3 * i] = RGBA[4 * X - 4 * i - 4];
      RGB[3 * i + 1] = RGBA[4 * X - 4 * i - 3];
      RGB[3 * i + 2] = RGBA[4 * X - 4 * i - 2];
    }
    lcd.pushImage(x, y + row, X, 1, (lgfx::rgb888_t *)RGB);
  }

  f.close();
  return 0;
}



void setup()
{
  
  Serial.begin(115200);

  
  pinMode(LCD_BLK, OUTPUT);
  
  
  
  //OLED
  u8g2.begin();
  u8g2.enableUTF8Print();        // enable UTF8 support for the Arduino print() function
  u8g2.setFont(u8g2_font_ncenB14_tr);
  u8g2.setFontDirection(0);
  Wire.end();
  //灯
  pinMode(40, OUTPUT);
  digitalWrite(40, LOW);
  ticker1.attach(0, callback1);

  
  lcd.init();
  //lcd.setSwapBytes(true);
  lcd.fillScreen(TFT_BLACK);
  delay(500);
  digitalWrite(LCD_BLK, HIGH);
  lcd.setTextColor(TFT_RED);
  lcd.setTextSize(2);
  lcd.setTextFont(2);
  
  //Buzzer
  pinMode(20, OUTPUT);
  ledcSetup(4, 5000, 8);
  ledcAttachPin(20, 4);
  print_img(SD, "/1.bmp", 480, 320);
  delay(2000);
}



void loop()
{
    Serial.println("Display TF card photos");
    //lcd.fillScreen(TFT_BLACK);
    //lcd.setRotation(7);
   // lcd.fillScreen(TFT_BLACK);
}
